@@ -1,3 +1,4 @@
+import io
 import construct as cs
 import construct_typed as cst
 import typing as t
@@ -5,9 +6,11 @@ import copy
 
 
 class GuiMetaData(t.TypedDict):
-    byte_range: t.Optional[t.Tuple[int, int]]
+    byte_range: t.Tuple[int, int]
     construct: cs.Construct
     context: "cs.Context"
+    stream: io.BytesIO
+    child_gui_metadata: t.Optional["GuiMetaData"]
 
 
 class IntWithGuiMetadata(int):
@@ -74,19 +77,28 @@ def add_gui_metadata(obj: t.Any, gui_metadata: GuiMetaData) -> t.Any:
 class IncludeGuiMetaData(cs.Subconstruct):
     """Include GUI metadata to the parsed object"""
 
-    def __init__(self, subcon, stream_nesting: int):
+    def __init__(self, subcon, bitwise: bool):
         super().__init__(subcon)  # type: ignore
-        self.stream_nesting = stream_nesting
+        self.bitwise = bitwise
 
     def _parse(self, stream, context, path):
         offset_start = cs.stream_tell(stream, path)
         obj = self.subcon._parsereport(stream, context, path)  # type: ignore
         offset_end = cs.stream_tell(stream, path)
 
+        # Maybe the obj has already gui_metadata. Read it
+        # out and save it in the parent gui_metadata object.
+        child_gui_metadata = get_gui_metadata(obj)
+
+        if self.bitwise is True:
+            stream._construct_bitstream_flag = True
+
         gui_metadata = GuiMetaData(
-            byte_range=(offset_start, offset_end) if self.stream_nesting == 0 else None,
+            byte_range=(offset_start, offset_end),
             construct=self.subcon,
             context=context,
+            stream=stream,
+            child_gui_metadata=child_gui_metadata,
         )
 
         return add_gui_metadata(obj, gui_metadata)
@@ -96,13 +108,16 @@ class IncludeGuiMetaData(cs.Subconstruct):
         return obj
 
 
+# #############################################################################
 def include_metadata(
-    constr: "cs.Construct[t.Any, t.Any]", stream_nesting: int = 0
+    constr: "cs.Construct[t.Any, t.Any]", bitwise: bool = False
 ) -> "cs.Construct[t.Any, t.Any]":
     """
     Surrond all named entries of a construct with offsets, so that
     we know the offset in the byte-stream and the length
     """
+
+    # ########## Simple Constructs ############################################
     if isinstance(
         constr,
         (
@@ -122,58 +137,26 @@ def include_metadata(
             cs.Seek,
         ),
     ):
-        return IncludeGuiMetaData(constr, stream_nesting)
+        return IncludeGuiMetaData(constr, bitwise)
 
-    elif isinstance(
-        constr,
-        (
-            cs.Restreamed,
-            cs.Transformed,
-            cs.Tunnel,
-            cs.Prefixed,
-            cs.FixedSized,
-            cs.NullStripped,
-        ),
+    # ########## Bitwiese Construct ###########################################
+    elif (
+        isinstance(constr, (cs.Restreamed))
+        and (constr.decoder is cs.bytes2bits)
+        and (constr.encoder is cs.bits2bytes)
+    ) or (
+        isinstance(constr, (cs.Transformed))
+        and (constr.decodefunc is cs.bytes2bits)
+        and (constr.encodefunc is cs.bits2bytes)
     ):
-        # these constructs manipulate the stream, so the offsets of all nested subcons are wrong.
-        # to detect this a counter is incremented for every nested stream
         constr = copy.copy(constr)  # constr is modified, so we have to make a copy
-        constr.subcon = include_metadata(constr.subcon, stream_nesting + 1)  # type: ignore
-        return IncludeGuiMetaData(constr, stream_nesting)
-
-    elif isinstance(constr, (cs.Struct)):
-        constr = copy.copy(constr)  # constr is modified, so we have to make a copy
-        new_subcons = []
-        for subcon in constr.subcons:
-            new_subcons.append(include_metadata(subcon, stream_nesting))
-        constr.subcons = new_subcons
-        return IncludeGuiMetaData(constr, stream_nesting)
-
-    elif isinstance(constr, (cs.Array, cs.GreedyRange)):
-        constr = copy.copy(constr)  # constr is modified, so we have to make a copy
-        constr.subcon = include_metadata(constr.subcon, stream_nesting)
-        return IncludeGuiMetaData(constr, stream_nesting)
-
-    elif isinstance(constr, cs.IfThenElse):
-        constr = copy.copy(constr)  # constr is modified, so we have to make a copy
-        constr.thensubcon = include_metadata(constr.thensubcon, stream_nesting)
-        constr.elsesubcon = include_metadata(constr.elsesubcon, stream_nesting)
-        return IncludeGuiMetaData(constr, stream_nesting)
-
-    elif isinstance(constr, cs.Switch):
-        constr = copy.copy(constr)  # constr is modified, so we have to make a copy
-        new_cases = {}
-        for key, subcon in constr.cases.items():
-            new_cases[key] = include_metadata(subcon, stream_nesting)
-        constr.cases = new_cases
-        if constr.default is not None:
-            constr.default = include_metadata(constr.default, stream_nesting)
-        return IncludeGuiMetaData(constr, stream_nesting)
+        constr.subcon = include_metadata(constr.subcon, bitwise=True)
+        return IncludeGuiMetaData(constr, bitwise)
 
+    # ########## Subconstructs ################################################
     elif isinstance(
         constr,
         (
-            cs.Renamed,
             cs.Const,
             cs.Rebuild,
             cs.Default,
@@ -181,17 +164,80 @@ def include_metadata(
             cs.Aligned,
             cs.Pointer,
             cs.Peek,
+            cst.DataclassStruct,
+            cs.Array,
+            cs.GreedyRange,
+            cs.Restreamed,
+            cs.Transformed,
+            cs.Tunnel,
+            cs.Prefixed,
+            cs.FixedSized,
+            cs.NullStripped,
+            cs.NullTerminated,
         ),
     ):
         constr = copy.copy(constr)  # constr is modified, so we have to make a copy
-        constr.subcon = include_metadata(constr.subcon, stream_nesting)  # type: ignore
+        constr.subcon = include_metadata(constr.subcon, bitwise)  # type: ignore
+        return IncludeGuiMetaData(constr, bitwise)
+
+    # Struct ##################################################################
+    elif isinstance(constr, cs.Struct):
+        constr = copy.copy(constr)  # constr is modified, so we have to make a copy
+        new_subcons = []
+        for subcon in constr.subcons:
+            new_subcons.append(include_metadata(subcon, bitwise))
+        constr.subcons = new_subcons
+        return IncludeGuiMetaData(constr, bitwise)
+
+    # FocusedSeq ##############################################################
+    elif isinstance(constr, cs.FocusedSeq):
+        constr = copy.copy(constr)  # constr is modified, so we have to make a copy
+        new_subcons = []
+        for subcon in constr.subcons:
+            new_subcons.append(include_metadata(subcon, bitwise))
+        constr.subcons = new_subcons
+        return IncludeGuiMetaData(constr, bitwise)
+
+    # Select ##################################################################
+    elif isinstance(constr, cs.Select):
+        constr = copy.copy(constr)  # constr is modified, so we have to make a copy
+        new_subcons = []
+        for subcon in constr.subcons:
+            new_subcons.append(include_metadata(subcon, bitwise))
+        constr.subcons = new_subcons
+        return IncludeGuiMetaData(constr, bitwise)
+
+    # IfThenElse ##############################################################
+    elif isinstance(constr, cs.IfThenElse):
+        constr = copy.copy(constr)  # constr is modified, so we have to make a copy
+        constr.thensubcon = include_metadata(constr.thensubcon, bitwise)
+        constr.elsesubcon = include_metadata(constr.elsesubcon, bitwise)
+        return IncludeGuiMetaData(constr, bitwise)
+
+    # Switch ##################################################################
+    elif isinstance(constr, cs.Switch):
+        constr = copy.copy(constr)  # constr is modified, so we have to make a copy
+        new_cases = {}
+        for key, subcon in constr.cases.items():
+            new_cases[key] = include_metadata(subcon, bitwise)
+        constr.cases = new_cases
+        if constr.default is not None:
+            constr.default = include_metadata(constr.default, bitwise)
+        return IncludeGuiMetaData(constr, bitwise)
+
+    # Checksum #################################################################
+    elif isinstance(constr, cs.Checksum):
+        constr = copy.copy(constr)  # constr is modified, so we have to make a copy
+        constr.checksumfield = include_metadata(constr.checksumfield, bitwise)
+        return IncludeGuiMetaData(constr, bitwise)
+
+    # Renamed #################################################################
+    elif isinstance(constr, cs.Renamed):
+        constr = copy.copy(constr)  # constr is modified, so we have to make a copy
+        constr.subcon = include_metadata(constr.subcon, bitwise)  # type: ignore
         return constr
 
-    elif isinstance(constr, cst.DataclassStruct):
-        constr = copy.copy(constr)  # constr is modified, so we have to make a copy
-        constr.subcon = include_metadata(constr.subcon, stream_nesting)  # type: ignore
-        return IncludeGuiMetaData(constr, stream_nesting)
-
+    # Misc ####################################################################
     elif isinstance(
         constr,
         (
@@ -208,16 +254,14 @@ def include_metadata(
             type(cs.Tell),
             type(cs.Pass),
             type(cs.Terminated),
-        ),  # type: ignore
+        ),
     ):
-        return IncludeGuiMetaData(constr, stream_nesting)
+        return IncludeGuiMetaData(constr, bitwise)
 
     # TODO:
     # # Grouping:
     # - Sequence
-    # - FocusedSeq
     # - Union
-    # - Select
     # - LazyStruct
 
     # # Grouping lists:
