@@ -14,9 +14,15 @@ import construct_editor.gallery.example_ipstack
 import construct_editor.gallery.test_bytes_greedybytes
 import construct_editor.gallery.test_array
 import construct_editor.gallery.test_greedyrange
+import construct_editor.gallery.test_bitwise
+import construct_editor.gallery.test_bits_swapped_bitwise
 import construct_editor.gallery.test_renamed
+import construct_editor.gallery.test_select
+import construct_editor.gallery.test_select_complex
 import construct_editor.gallery.test_ifthenelse
+import construct_editor.gallery.test_ifthenelse_nested_switch
 import construct_editor.gallery.test_switch
+import construct_editor.gallery.test_switch_dataclass
 import construct_editor.gallery.test_dataclass_struct
 import construct_editor.gallery.test_dataclass_bit_struct
 import construct_editor.gallery.test_flag
@@ -24,13 +30,21 @@ import construct_editor.gallery.test_enum
 import construct_editor.gallery.test_flagsenum
 import construct_editor.gallery.test_tenum
 import construct_editor.gallery.test_tflagsenum
+import construct_editor.gallery.test_const
 import construct_editor.gallery.test_computed
+import construct_editor.gallery.test_focusedseq
 import construct_editor.gallery.test_timestamp
 import construct_editor.gallery.test_padded
 import construct_editor.gallery.test_aligned
 import construct_editor.gallery.test_pointer_peek_seek_tell
 import construct_editor.gallery.test_pass
 import construct_editor.gallery.test_fixedsized
+import construct_editor.gallery.test_nullstripped
+import construct_editor.gallery.test_nullterminated
+import construct_editor.gallery.test_checksum
+import construct_editor.gallery.test_compressed
+import construct_editor.gallery.test_stringencodded
+import construct_editor.gallery.example_cmd_resp
 from construct_editor.widgets.construct_hex_editor import ConstructHexEditor
 
 
@@ -57,6 +71,7 @@ class ConstructGallery(wx.Panel):
             "################ EXAMPLES ################": None,
             "Example: pe32coff": construct_editor.gallery.example_pe32coff.gallery_item,
             "Example: ipstack": construct_editor.gallery.example_ipstack.gallery_item,
+            "Example: Cmd/Resp": construct_editor.gallery.example_cmd_resp.gallery_item,
             "################ TESTS ####################": None,
             # "## bytes and bits ################": None,
             "Test: Bytes/GreedyBytes": construct_editor.gallery.test_bytes_greedybytes.gallery_item,
@@ -64,8 +79,10 @@ class ConstructGallery(wx.Panel):
             # "Test: FormatField (TODO)": None,
             # "Test: BytesInteger (TODO)": None,
             # "Test: BitsInteger (TODO)": None,
-            # "## strings #######################": None,
-            # "Test: StringEncoded (TODO)": None,
+            "Test: Bitwiese": construct_editor.gallery.test_bitwise.gallery_item,
+            "Test: BitsSwapped/Bitwiese": construct_editor.gallery.test_bits_swapped_bitwise.gallery_item,
+            "## strings #######################": None,
+            "Test: StringEncoded": construct_editor.gallery.test_stringencodded.gallery_item,
             "## mappings ######################": None,
             "Test: Flag": construct_editor.gallery.test_flag.gallery_item,
             "Test: Enum": construct_editor.gallery.test_enum.gallery_item,
@@ -85,14 +102,14 @@ class ConstructGallery(wx.Panel):
             "## specials ##########################": None,
             "Test: Renamed": construct_editor.gallery.test_renamed.gallery_item,
             "## miscellaneous ##########################": None,
-            # "Test: Const (TODO)": None,
+            "Test: Const": construct_editor.gallery.test_const.gallery_item,
             "Test: Computed": construct_editor.gallery.test_computed.gallery_item,
             # "Test: Index (TODO)": None,
             # "Test: Rebuild (TODO)": None,
             # "Test: Default (TODO)": None,
             # "Test: Check (TODO)": None,
             # "Test: Error (TODO)": None,
-            # "Test: FocusedSeq (TODO)": None,
+            "Test: FocusedSeq": construct_editor.gallery.test_focusedseq.gallery_item,
             # "Test: Pickled (TODO)": None,
             # "Test: Numpy (TODO)": None,
             # "Test: NamedTuple (TODO)": None,
@@ -101,9 +118,12 @@ class ConstructGallery(wx.Panel):
             # "Test: HexDump (TODO)": None,
             "## conditional ##########################": None,
             # "Test: Union (TODO)": None,
-            # "Test: Select (TODO)": None,
+            "Test: Select": construct_editor.gallery.test_select.gallery_item,
+            "Test: Select (Complex)": construct_editor.gallery.test_select_complex.gallery_item,
             "Test: IfThenElse": construct_editor.gallery.test_ifthenelse.gallery_item,
+            "Test: IfThenElse (Nested Switch)": construct_editor.gallery.test_ifthenelse_nested_switch.gallery_item,
             "Test: Switch": construct_editor.gallery.test_switch.gallery_item,
+            "Test: Switch (Dataclass)": construct_editor.gallery.test_switch_dataclass.gallery_item,
             # "Test: StopIf (TODO)": None,
             "## alignment and padding ##########################": None,
             "Test: Padded": construct_editor.gallery.test_padded.gallery_item,
@@ -116,15 +136,15 @@ class ConstructGallery(wx.Panel):
             # "Test: RawCopy (TODO)": None,
             # "Test: Prefixed (TODO)": None,
             "Test: FixedSized": construct_editor.gallery.test_fixedsized.gallery_item,
-            # "Test: NullTerminated (TODO)": None,
-            # "Test: NullStripped (TODO)": None,
+            "Test: NullTerminated": construct_editor.gallery.test_nullterminated.gallery_item,
+            "Test: NullStripped": construct_editor.gallery.test_nullstripped.gallery_item,
             # "Test: RestreamData (TODO)": None,
             # "Test: Transformed (TODO)": None,
             # "Test: Restreamed (TODO)": None,
             # "Test: ProcessXor (TODO)": None,
             # "Test: ProcessRotateLeft (TODO)": None,
-            # "Test: Checksum (TODO)": None,
-            # "Test: Compressed (TODO)": None,
+            "Test: Checksum": construct_editor.gallery.test_checksum.gallery_item,
+            "Test: Compressed": construct_editor.gallery.test_compressed.gallery_item,
             # "Test: CompressedLZ4 (TODO)": None,
             # "Test: Rebuffered (TODO)": None,
             # "## lazy equivalents ##########################": None,
@@ -273,6 +293,7 @@ class ConstructGallery(wx.Panel):
     def on_clear_binary_clicked(self, event):
         self.example_selector_lbx.SetSelection(wx.NOT_FOUND)
         self.construct_hex_editor.binary = bytes()
+        self.construct_hex_editor.construct_editor.expand_all()
 
     def on_example_selection_changed(self, event):
         selection = self.gallery_selector_lbx.GetStringSelection()
@@ -281,6 +302,7 @@ class ConstructGallery(wx.Panel):
 
         # Set example binary
         self.construct_hex_editor.binary = example_binary
+        self.construct_hex_editor.construct_editor.expand_all()
 
     def on_load_binary_file_clicked(self, event):
         with wx.FileDialog(
@@ -370,12 +392,15 @@ def main():
 
         app = wit.InspectableApp()
     else:
+        wit = None
         app = wx.App(False)
 
     frame = ConstructGalleryFrame(None)
     frame.Show(True)
+    if wit is not None:
+        wit.InspectionTool().Show()
     app.MainLoop()
 
 
 if __name__ == "__main__":
-    main()
\ No newline at end of file
+    main()
