@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 import enum
 from typing import Any, Callable, Dict, List, Optional
-
+import typing as t
 import construct as cs
 import wx
 import wx.dataview as dv
@@ -10,7 +10,7 @@ import textwrap
 
 from construct_editor.helper.preprocessor import include_metadata
 from construct_editor.helper.wrapper import (
-    ObjPanel,
+    ObjEditorMixin,
     EntryConstruct,
     create_entry_from_construct,
     get_gui_metadata,
@@ -78,12 +78,12 @@ class ObjectRenderer(dv.DataViewCustomRenderer):
         return True
 
     def CreateEditorCtrl(self, parent, labelRect: wx.Rect, value: EntryConstruct):
-        editor: ObjPanel = value.create_obj_panel(parent)
+        editor: ObjEditorMixin = value.create_obj_panel(parent)
         editor.SetPosition(labelRect.Position)
         editor.SetSize(labelRect.Size)
         return editor
 
-    def GetValueFromEditorCtrl(self, editor: ObjPanel):
+    def GetValueFromEditorCtrl(self, editor: ObjEditorMixin):
         new_obj = editor.get_new_obj()
         # new_obj is passed in a dict, because subtypes of `int` (eg. `IntEnum`)
         # and `str` (eg. `cs.EnumIntegerString`) are converted to its base type
@@ -160,7 +160,7 @@ class ContextMenu(wx.Menu):
             submenu = wx.Menu()
             self.submenu_map: Dict[Any, EntryConstruct] = {}
             for e in model.list_viewed_entries:
-                name = "->".join(e.path)
+                name = ".".join(e.path)
                 item: wx.MenuItem = submenu.AppendCheckItem(wx.ID_ANY, name)
                 self.submenu_map[item.GetId()] = e
                 self.Bind(wx.EVT_MENU, self.on_remove_list_viewed_item, item)
@@ -256,7 +256,7 @@ class ConstructEditorModel(dv.PyDataViewModel):
         flat_subentry_list: List["EntryConstruct"] = []
         entry.create_flat_subentry_list(flat_subentry_list)
 
-        return "->".join(flat_subentry_list[col].path)
+        return ".".join(flat_subentry_list[col].path)
 
     # #################################################################################################################
     # dv.PyDataViewModel Interface ####################################################################################
@@ -330,10 +330,7 @@ class ConstructEditorModel(dv.PyDataViewModel):
 
         # return parent of entry
         entry: EntryConstruct = self.ItemToObject(item)
-        if entry.parent is None:
-            return dv.NullDataViewItem
-        else:
-            return entry.parent.dvc_item
+        return entry.get_parent_dvc_item()
 
     def GetValue(self, item: dv.DataViewItem, col: int):
         # Return the value to be displayed for this item and column.
@@ -436,7 +433,7 @@ class ConstructEditor(wx.Panel):
             style=dv.DV_VERT_RULES | dv.DV_ROW_LINES,
             name="construct_editor",
         )
-        vsizer.Add(self._dvc, 3, wx.ALL | wx.EXPAND, 5)
+        vsizer.Add(self._dvc, 3, wx.EXPAND, 0)
 
         # Create Model of DataViewCtrl
         self._model = ConstructEditorModel(self._dvc)
@@ -478,7 +475,9 @@ class ConstructEditor(wx.Panel):
         self._dvc_main_window: wx.Window = self._dvc.GetMainWindow()
         self._dvc_main_window.Bind(wx.EVT_MOTION, self._on_dvc_motion)
         self._dvc_main_window.Bind(wx.EVT_KEY_DOWN, self._on_dvc_key_down)
-        self._last_motion_obj: Optional[EntryConstruct] = None
+        self._last_tooltip: Optional[
+            t.Tuple[EntryConstruct, ConstructEditorColumn]
+        ] = None
 
         self._dvc_header = self._dvc.FindWindowByName("wxMSWHeaderCtrl")
         if self._dvc_header is not None:
@@ -587,6 +586,10 @@ class ConstructEditor(wx.Panel):
         self._model.hide_protected = value
         self.reload()
 
+    # expand_entry ############################################################
+    def expand_entry(self, entry: EntryConstruct):
+        self._dvc.Expand(entry.dvc_item)
+
     # expand_children #########################################################
     def expand_children(self, entry: EntryConstruct):
         if entry.subentries is not None:
@@ -689,7 +692,7 @@ class ConstructEditor(wx.Panel):
             if list_col < len(flat_list):
                 path = flat_list[list_col].path
                 path = path[len(entry.path) :]  # remove the path from the parent
-                dvc_column.SetTitle("->".join(path))
+                dvc_column.SetTitle(".".join(path))
             else:
                 dvc_column.SetTitle(str(list_col))
 
@@ -712,15 +715,18 @@ class ConstructEditor(wx.Panel):
             self._clear_status_bar()
 
     def _refresh_status_bar(self, entry: EntryConstruct):
-        self._status_bar.SetStatusText("->".join(entry.path), 0)
+        self._status_bar.SetStatusText(".".join(entry.path), 0)
         bytes_info = ""
-        metadata = entry.obj_metadata
-        if (metadata is not None) and (metadata["byte_range"] is not None):
-            start = metadata["byte_range"][0]
-            end = metadata["byte_range"][1] - 1
+        stream_infos = entry.get_stream_infos()
+
+        # Show byte range only when no nested streams are used
+        if len(stream_infos) == 1:
+            byte_range = stream_infos[0].byte_range
+            start = byte_range[0]
+            end = byte_range[1] - 1
             size = end - start + 1
             if size > 0:
-                bytes_info = f"Bytes: {start}-{end} ({size})"
+                bytes_info = f"Bytes: {start:n}-{end:n} ({size:n})"
         self._status_bar.SetStatusText(bytes_info, 1)
 
     def _clear_status_bar(self):
@@ -741,16 +747,21 @@ class ConstructEditor(wx.Panel):
         if item.GetID() is None:
             self._dvc_main_window.SetToolTip("")
             return
-        obj: EntryConstruct = self._model.ItemToObject(item)
+        entry: EntryConstruct = self._model.ItemToObject(item)
 
         if col.ModelColumn == ConstructEditorColumn.Name:
             # only set tooltip if the obj changed. this prevents flickering
-            if self._last_motion_obj is not obj:
-                self._dvc_main_window.SetToolTip(textwrap.dedent(obj.docs).strip())
-            self._last_motion_obj = obj
+            if self._last_tooltip != (entry, ConstructEditorColumn.Name):
+                self._dvc_main_window.SetToolTip(textwrap.dedent(entry.docs).strip())
+            self._last_tooltip = (entry, ConstructEditorColumn.Name)
+        elif col.ModelColumn == ConstructEditorColumn.Type:
+            # only set tooltip if the obj changed. this prevents flickering
+            if self._last_tooltip != (entry, ConstructEditorColumn.Type):
+                self._dvc_main_window.SetToolTip(str(entry.construct))
+            self._last_tooltip = (entry, ConstructEditorColumn.Type)
         else:
             self._dvc_main_window.SetToolTip("")
-            self._last_motion_obj = None
+            self._last_tooltip = None
 
     def _on_dvc_header_motion(self, event: wx.MouseEvent):
         event.Skip()  # TODO: Create Tooltip for DVC-Header
