@@ -37,17 +37,14 @@ class HexEditorBinaryData:
     This class is used mainly to track changes and notiy everyone
     """
 
-    def __init__(self, binary: Optional[bytes] = None) -> None:
-        if binary is None:
-            self._binary = bytearray()
-        else:
-            self._binary = bytearray(binary)
+    def __init__(self, binary: bytes) -> None:
+        self._binary = bytearray(binary)
 
         self.on_binary_changed = BinaryChangedCallbackList()
         self.command_processor = wx.CommandProcessor()
 
     def overwrite_all(self, byts: bytes):
-        """ overwrite the complete data with the new ones """
+        """overwrite the complete data with the new ones"""
         obj = self
 
         class Cmd(wx.Command):
@@ -70,7 +67,7 @@ class HexEditorBinaryData:
         self.command_processor.Submit(Cmd())
 
     def overwrite_range(self, idx: int, byts: bytes):
-        """ overwrite byte range beginning from the given index """
+        """overwrite byte range beginning from the given index"""
         obj = self
 
         class Cmd(wx.Command):
@@ -95,7 +92,7 @@ class HexEditorBinaryData:
         self.command_processor.Submit(Cmd())
 
     def insert_range(self, idx: int, byts: bytes):
-        """ inserts byte range at the given index """
+        """inserts byte range at the given index"""
         obj = self
 
         class Cmd(wx.Command):
@@ -117,7 +114,7 @@ class HexEditorBinaryData:
         self.command_processor.Submit(Cmd())
 
     def remove_range(self, idx: int, length: int):
-        """ remove the bytes at at the given range """
+        """remove the bytes at at the given range"""
         obj = self
 
         class Cmd(wx.Command):
@@ -139,15 +136,15 @@ class HexEditorBinaryData:
         self.command_processor.Submit(Cmd())
 
     def get_value(self, idx: int):
-        """ get the value at the given index """
+        """get the value at the given index"""
         return self._binary[idx]
 
     def get_range(self, idx: int, len: int):
-        """ get the value at the given index """
+        """get the value at the given index"""
         return bytes(self._binary[idx : idx + len])
 
     def get_bytes(self) -> bytes:
-        """ return readonly version of the data """
+        """return readonly version of the data"""
         return bytes(self._binary)
 
     def __len__(self):
@@ -210,6 +207,12 @@ class HexEditorTable(Grid.GridTableBase):
         idx = (row * self._cols) + col
         return idx
 
+    def refresh_rows_cols(self):
+        self._rows = math.ceil(len(self._binary_data) / self._editor.format.width)
+        if (len(self._binary_data) % self._editor.format.width) == 0:
+            self._rows += 1
+        self._cols = self._editor.format.width
+
     # ############################################ GridTableBase Interface ############################################
     def SetValue(self, row: int, col: int, value: str):
         byte_idx = self.get_byte_idx(row, col)
@@ -608,11 +611,14 @@ class HexEditorGrid(Grid.Grid):
         editor: "HexEditor",
         table: HexEditorTable,
         binary_data: HexEditorBinaryData,
+        read_only: bool = False,
     ):
         super().__init__(editor)
         self._editor = editor
         self._table = table
         self._binary_data = binary_data
+        self.read_only = read_only
+        self.on_selection_changed = SelectionChangedCallbackList()
 
         # The second parameter means that the grid is to take
         # ownership of the table and will destroy it when done.
@@ -624,6 +630,7 @@ class HexEditorGrid(Grid.Grid):
         self.EnableDragGridSize(False)
         self.EnableDragRowSize(False)
         self.EnableDragColSize(False)
+        self.EnableEditing(not self.read_only)
 
         self.SetRowLabelSize(50)
         self.SetColLabelSize(20)
@@ -644,34 +651,31 @@ class HexEditorGrid(Grid.Grid):
         self.refresh()
 
         self._selection: t.Tuple[Optional[int], Optional[int]] = (None, None)
-        self.on_selection_changed = SelectionChangedCallbackList()
 
     def refresh(self):
         """
         (Grid) -> Reset the grid view.   Call this to
         update the grid if rows and columns have been added or deleted
         """
-        oldrows = self._table._rows
-        oldcols = self._table._cols
+        oldrows = self._table.GetNumberRows()
+        oldcols = self._table.GetNumberCols()
 
-        self._table._rows = math.ceil(
-            len(self._binary_data) / self._editor.format.width
-        )
-        # if (len(self._binary_data) % self._editor.format.width) == 1:
-        #     self._table._rows += 1
-        self._table._cols = self._editor.format.width
+        self._table.refresh_rows_cols()
+
+        newrows = self._table.GetNumberRows()
+        newcols = self._table.GetNumberCols()
 
         self.BeginBatch()
         for current, new, delmsg, addmsg in [
             (
                 oldrows,
-                self._table._rows,
+                newrows,
                 Grid.GRIDTABLE_NOTIFY_ROWS_DELETED,
                 Grid.GRIDTABLE_NOTIFY_ROWS_APPENDED,
             ),
             (
                 oldcols,
-                self._table._cols,
+                newcols,
                 Grid.GRIDTABLE_NOTIFY_COLS_DELETED,
                 Grid.GRIDTABLE_NOTIFY_COLS_APPENDED,
             ),
@@ -700,7 +704,7 @@ class HexEditorGrid(Grid.Grid):
 
         # settings for each column
         hexcol_width = (char_width * 2) + 5
-        for col in range(self._table._cols):
+        for col in range(newcols):
             logger.debug("hexcol %d width=%d" % (col, hexcol_width))
             self.SetColMinimalWidth(col, 0)
             self.SetColSize(col, hexcol_width)
@@ -731,14 +735,14 @@ class HexEditorGrid(Grid.Grid):
             event.Skip()
 
     def _on_select_cell(self, event: Grid.GridEvent):
-        """ Single cell selected """
+        """Single cell selected"""
         idx = self._table.get_byte_idx(event.GetRow(), event.GetCol())
         self.ClearSelection()
         self._selection = (idx, None)
         self.on_selection_changed.fire(start_idx=idx, end_idx=None)
 
     def _on_range_selecting_mouse(self, event):
-        """ Change selection from a rectangular block to a range between two indexes """
+        """Change selection from a rectangular block to a range between two indexes"""
         # get the first selected item
         row1, col1 = self.GetGridCursorCoords()
 
@@ -755,7 +759,7 @@ class HexEditorGrid(Grid.Grid):
         self.select_range(idx1, idx2)
 
     def _on_range_selecting_keyboard(self, row_diff: int = 0, col_diff: int = 0):
-        """ Change selection from the keyboard """
+        """Change selection from the keyboard"""
         sel = self._selection
         if sel[0] is None:
             return  # nothing is currently selected
@@ -780,7 +784,7 @@ class HexEditorGrid(Grid.Grid):
         self.select_range(cursor_idx, other_idx)
 
     def select_range(self, idx1: int, idx2: int):
-        """ Select the range between two byte indexes. """
+        """Select the range between two byte indexes."""
         if idx1 < 0 or idx2 < 0:
             return
         idx1 = min(idx1, len(self._binary_data) - 1)
@@ -810,13 +814,34 @@ class HexEditorGrid(Grid.Grid):
         self._selection = (idx1, idx2)
         self.on_selection_changed.fire(start_idx=idx1, end_idx=idx2)
 
-    def _cut_selection(self):
-        """ Copy the selected data to the clipboard and remove it from the binary data """
-        if self._copy_selection():
-            self._remove_selection()
+    def _cut_selection(self) -> bool:
+        """
+        Copy the selected data to the clipboard and remove it from the binary data
+
+        Return:
+         - true if copy is okay
+         - false if an error occured
+        """
+        if self.read_only is True:
+            return False
+
+        if self._copy_selection() is False:
+            return False
+        if self._remove_selection() is False:
+            return False
+        return True
 
     def _remove_selection(self) -> bool:
-        """ Remove the selected bytes """
+        """
+        Remove the selected bytes
+
+        Return:
+         - true if copy is okay
+         - false if an error occured
+        """
+        if self.read_only is True:
+            return False
+
         sel = self._selection
         if sel[0] is None:
             return False
@@ -835,7 +860,13 @@ class HexEditorGrid(Grid.Grid):
         return True
 
     def _copy_selection(self) -> bool:
-        """ Copy the selected data to the clipboard """
+        """
+        Copy the selected data to the clipboard
+
+        Return:
+         - true if copy is okay
+         - false if an error occured
+        """
         sel = self._selection
         if sel[0] is None:
             return False
@@ -866,6 +897,9 @@ class HexEditorGrid(Grid.Grid):
         If insert=True: Insert new bytes to the binary data. The binary data is always
                         increased by the size of the data from the clipboard.
         """
+        if self.read_only is True:
+            return False
+
         # check if somethis is selected
         sel = self._selection
         if sel[0] is None:
@@ -978,7 +1012,7 @@ class HexEditorGrid(Grid.Grid):
             event.Skip()
 
     def _on_cell_right_click(self, event: Grid.GridEvent):
-        """ Show context menu """
+        """Show context menu"""
         # Check if the click is inside the current selection.
         # If not, select the current cell
         sel = self._selection
@@ -992,19 +1026,24 @@ class HexEditorGrid(Grid.Grid):
             self.SetGridCursor(event.GetRow(), event.GetCol())
 
         menus = [
-            (wx.ID_CUT, "Cut\tCtrl+X", lambda event: self._cut_selection(), True),
+            (
+                wx.ID_CUT,
+                "Cut\tCtrl+X",
+                lambda event: self._cut_selection(),
+                not self.read_only,
+            ),
             (wx.ID_COPY, "Copy\tCtrl+C", lambda event: self._copy_selection(), True),
             (
                 wx.ID_PASTE,
                 "Paste (overwrite)\tCtrl+V",
                 lambda event: self._paste(overwrite=True),
-                True,
+                not self.read_only,
             ),
             (
                 wx.ID_PASTE,
                 "Paste (insert)\tCtrl+Shift+V",
                 lambda event: self._paste(insert=True),
-                True,
+                not self.read_only,
             ),
             None,
             (
@@ -1045,8 +1084,10 @@ class HexEditor(wx.Panel):
     def __init__(
         self,
         parent,
-        binary: Optional[bytes] = None,
+        binary: bytes = b"",
         format: Optional[HexEditorFormat] = None,
+        read_only: bool = False,
+        bitwiese: bool = False,
     ):
         super().__init__(parent)
 
@@ -1055,13 +1096,14 @@ class HexEditor(wx.Panel):
             self._format = HexEditorFormat()
         else:
             self._format = format
+        self.bitwiese = bitwiese
 
         # self.control = wx.TextCtrl(self, style=wx.TE_MULTILINE)
         sizer = wx.BoxSizer(wx.VERTICAL)
 
         # create HexEditorTable & HexEditorGrid
         self._table = HexEditorTable(self, self._binary_data)
-        self._grid = HexEditorGrid(self, self._table, self._binary_data)
+        self._grid = HexEditorGrid(self, self._table, self._binary_data, read_only)
         sizer.Add(self._grid, 1, wx.ALL | wx.EXPAND, 0)
 
         # create status bar
@@ -1082,27 +1124,32 @@ class HexEditor(wx.Panel):
         self.Show(True)
 
     def _on_binary_changed(self, binary_data: HexEditorBinaryData):
-        msg = f"{len(binary_data):d} Bytes"
+        if self.bitwiese is True:
+            unit = "Bits"
+        else:
+            unit = "Bytes"
+
+        msg = f"{len(binary_data):n} {unit}"
         self._status_bar.SetStatusText(msg, 0)
         self.refresh()
 
     def _on_selection_changed(self, idx1: int, idx2: Optional[int]):
         if idx2 is None:
-            msg = f"Selection: {idx1:d}"
+            msg = f"Selection: {idx1:n}"
         else:
-            msg = f"Selection: {idx1:d}-{idx2:d} ({idx2-idx1+1:d})"
+            msg = f"Selection: {idx1:n}-{idx2:n} ({idx2-idx1+1:n})"
 
         self._status_bar.SetStatusText(msg, 1)
 
     def colorise(self, start: int, end: int, refresh: bool = True):
-        """ Colorize a byte range in the Hex Editor. Only a singe range can be colorized """
+        """Colorize a byte range in the Hex Editor. Only a singe range can be colorized"""
         self._table.selections = [(start, end)]
 
         if refresh:
             self.refresh()
 
     def scroll_to_idx(self, idx: int, refresh: bool = True):
-        """ Scroll to a specific byte index in the Hex Editor """
+        """Scroll to a specific byte index in the Hex Editor"""
         row, col = self._table.get_byte_rowcol(idx)
 
         self._grid.MakeCellVisible(row, col)
@@ -1111,17 +1158,18 @@ class HexEditor(wx.Panel):
             self.refresh()
 
     def refresh(self):
-        """ Refresh the Grid, when some values have canged """
+        """Refresh the Grid, when some values have canged"""
         self._grid.refresh()
 
     # Property: binary ########################################################
     @property
     def binary(self) -> bytes:
-        """ Binary data, that is shown in the HexEditor. """
+        """Binary data, that is shown in the HexEditor."""
         return self._binary_data.get_bytes()
 
     @binary.setter
     def binary(self, val: bytes):
+        self.colorise(0, 0, True)
         self._binary_data.overwrite_all(val)
         # clear all commands, when new data is set from external
         self._binary_data.command_processor.ClearCommands()
@@ -1129,7 +1177,7 @@ class HexEditor(wx.Panel):
     # Property: format ##################################################
     @property
     def format(self) -> HexEditorFormat:
-        """ Format of the HexEditor. """
+        """Format of the HexEditor."""
         return self._format
 
     @format.setter
@@ -1151,7 +1199,7 @@ class HexEditor(wx.Panel):
 if __name__ == "__main__":
 
     class MyFrame(wx.Frame):
-        """ We simply derive a new class of Frame. """
+        """We simply derive a new class of Frame."""
 
         def __init__(self, parent, title):
             wx.Frame.__init__(self, parent, title=title, size=(420, 800))
